%% Clearing workspace, closing open figures
close all
clear all
%clc

% NOTE: Set noise, sampling rate and more in constants/simulations_parameters.m

%% Magnetic constants
mathematical_constants;

%% Earth model
Earth_parameters;

%% HYPSO HSI-Mission Spacecraft
HYPSO_orbital_parameters;
HYPSO_satellite_dimensions; 
HYPSO_saturation_bounds;
HYPSO_mtq_parameters;

%% Perturbation model
perturbation_parameters;

%% Calculate orbit
calculate_initial_orbit;

%% Magnetorquer control
HYPSO_mtq_gains;

MTQ = 2;

if MTQ == 0
    disp('Magnetorquer is off')
elseif MTQ == 1
    disp('Magnetorquer is on, in detumbling mode')
elseif MTQ == 2
    disp('Magnetorquer is on, in momentum dumping mode')
end

%% Initial parameters

% Initial attitude
q_init = eul2quat([20*pi/180 0 10*pi/180], 'ZYX')';

% Initial angular velocity (body + orbit)
w_ibb0 = [0;0;0];%[0.1;0.4;0.9];% Shows initial tumbling 

% w_ibb0 = [0.1;0.4;0.9];%*10; % when testing detumbling

w_obb_init = [0;0;0];
%% Setpoints
USE_ESTIMATES = 0; % 0/1
OBSERVERCHOICE = 3; % 1-5
UNCERTAIN_WHEELS = 0;% 0/1
SAVE_PLOTS = 0; % 0/1

% Observers:
% 1 - SMO based on quaternion measurements only
% 2 - SMO for quaternion, IMU for w_ibb
% 3 - HOSMO for quaternion, IMU for w_ibb
% 4 - HOSMO based on quaternion
% 5++ - Not included
    
CONTROL_MODE = 2;

if CONTROL_MODE == 1
    disp('Mode: Slewing')
elseif CONTROL_MODE == 2
    disp('Mode: Pointing (switching references)')
elseif CONTROL_MODE == 3
    disp('Mode: Pointing (one reference)')
elseif CONTROL_MODE == 4
    disp('Mode: HYPSO mission')
end

PHI_0 = 1;
max_speed = deg2rad(0.7238); % Set slew rate in degrees->radians.
ALPHA_0 = [0;max_speed;0];%[0.005;-0.004;0.002];
theta_0 = [0; PHI_0; 0];
% w_obb_ref_t_0 = [ALPHA_0*sin(PHI_0);0;ALPHA_0*(1-cos(PHI_0))];
w_obb_ref_t_0 = [0;0;0];
alpha = [0;0;-1];

if CONTROL_MODE == 1
    THETA = norm(theta_0);
    E = theta_0./norm(theta_0);
    q_init = axang2quat([E', THETA])'; % Equivalent:  q_init = [cos(THETA/2); E*sin(THETA/2)];
end

if q_init(1) == 0
    eta_zero_init = 0;
else
    eta_zero_init = 1;
end
if q_init(1) >= 0
    eta_sign_init = 1;
else
    eta_sign_init = -1;
end

SLEWTIME = 57+8;
SLEWPREPARE = 90; % Prepare slew

% CONTROL_MODE == 2
% Attitude setpoint
q_ref_1 = eul2quat([-20*pi/180 1e-16 1e-16], 'ZYX')';%[1;0;0;0];
%q_ref_2 = eul2quat([1e-16 20*pi/180 -20*pi/180], 'ZYX')';
%q_ref_2 = [0 0 1 0]';
%q_ref_2 = [0.1204 0 0.9631 0.2408]';
q_ref_2 = [0.4467 0 0.0486 0.8934]';
% q_ref_2 = [1 0 0 0]';

% Angular velocity setpoint 
% HAS TO BE [0;0;0]; FOR POINTING
w_ob_ref_1 = [0;0;0];%[0;-0.1;0.3];% 
w_ob_ref_2 = [0;0;0];


if CONTROL_MODE == 3
    % Attitude setpoint
    q_ref_1 = eul2quat([1e-16 20*pi/180 -20*pi/180], 'ZYX')';
    q_ref_2 = eul2quat([-20*pi/180 1e-16 1e-16], 'ZYX')';%[1;0;0;0];
    % Angular velocity setpoint 
    % HAS TO BE [0;0;0]; FOR POINTING
    w_ob_ref_1 = [0;0;0];
    w_ob_ref_2 = [0;0;0];
end


TIME_REF_THRESHOLD_1 = 30;%40;60
TIME_REF_THRESHOLD_2 = TIME_REF_THRESHOLD_1+100;
REFERENCE_INTERVAL = 180;

TIME_REF_THRESHOLD_SLEWING = 30*2;
eta_dot_dot_max = 10;
eta_dot_max = 100;
omega_ni = 1;
zeta = 1;


%% Uncertainties in placement of wheels
% Applies some distortion relative to the known torque distribution matrix.
A_nom = A;
A_nom_pinv = pinv(A_nom);

if UNCERTAIN_WHEELS
    wheel_perturb = 1e-2;
    wheel_perturb = 1e-1;
    for k =1:4

        if k == 4
            A(2,k) = A(2,k)+wheel_perturb;
        elseif mod(k,2)==0
            A(:,k)=A(:,k)+wheel_perturb;
        else
            A(:,k)=A(:,k)-wheel_perturb;
        end
    end
%     A(:,1)=A(:,1)+wheel_perturb;
    A = normc(A);
    A_pinv = pinv(A);
end


%% Plotting options
bool_plot_orbit                 = false;
bool_plot_complete_orbit        = false;
bool_plot_attitude              = false;
bool_plot_attitude_w_satellite  = false;

skip_step       = 50;
min_tail        = 150;
wanted_steps    = 250; % Confusing name, even worse use. Should be fixed in the future


%% Controller parameters

% List of controllers used in the master's thesis(reaction wheels):
% 1 - PD controller
% 3 - SMC - Used in thesis
% 5 - LQR(offline)
% 10 - GSTA
% -1 - No controller

HYPSO_RW_gains;

controllerChoice = 10;

if controllerChoice     == -1
    disp('No controller on reaction wheels (no RW being used)')
elseif controllerChoice == 1
    disp('Reaction wheels using PD controller')
elseif controllerChoice == 2
    disp('Reaction wheels using Sliding Mode Controller, first order, from Jrgensen, Gravdahl')
elseif controllerChoice == 3
    disp('Reaction wheels using Sliding Mode Controller, first order, simple')
elseif controllerChoice == 4
    disp('Reaction wheels using Sliding Mode Controller, first order, simple slewing')
elseif controllerChoice == 5
    disp('Reaction wheels using LQR (offline)')
elseif controllerChoice == 6
    disp('Reaction wheels using P control, slewing')
elseif controllerChoice == 7
    disp('Reaction wheels using PD control, slewing')
elseif controllerChoice == 10
    disp('Reaction wheels using Sliding Mode Controller, second order (GSTA)')
elseif controllerChoice == 11
    disp('Reaction wheels using Sliding Mode Controller, second order (GSTA). Compensates for one of the HOSMOs')
end


%% Observer
HYPSO_observer_gains;

%% Simulation

simulation_parameters;

sim('top_model')

%% Plotting
if USE_ESTIMATES
    if CONTROL_MODE ~= 1
        %control_point_plots(q_error, w_obb, w_ob_ref_ws, q_ob_real, q_ob_ref_ws);
        control_point_plots(q_error, HOSMO_w_obb_hat, w_ob_ref_ws, HOSMO_q_ob_hat, q_ob_ref_ws);
    else
        control_slewing_plots(q_error, HOSMO_w_obb_hat, w_ob_ref_ws, HOSMO_q_ob_hat, q_ob_ref_ws);
    end
else
    if CONTROL_MODE ~= 1
        control_point_plots(q_error, w_obb, w_ob_ref_ws, q_ob_real, q_ob_ref_ws);
    else
        control_slewing_plots(q_error, w_obb, w_ob_ref_ws, q_ob_real, q_ob_ref_ws);
    end
end

control_input_plots(w_bww, tau_uw, tau_mtq)
% SMO_plot(SMO_epsilon_hat, SMO_omega_hat, SMO_normalized_omega_error_norm, SMO_normalized_epsilon_error_norm, w_obb, q_ob)
HOSMO_plot(HOSMO_q_ob_hat, HOSMO_w_obb_hat, w_obb, w_obb_meas, q_ob_real, q_ob_meas)


if CONTROL_MODE == 3
    REF_TIME     = TIME_REF_THRESHOLD_1;
    point_tol    = 0.5;%0.5;               % Angular degrees. Defines the error band. 
    [settling_times, accuracies, ss_errors]    = system_characteristics(q_ob_real, q_ob_ref_ws, w_obb, w_ob_ref_ws, REF_TIME, CONTROL_MODE, point_tol)
elseif CONTROL_MODE == 1
    REF_TIME     = TIME_REF_THRESHOLD_SLEWING;
    slew_tol    = 4e-4;               % Defines the error band. 
    [settling_times, accuracies, ss_errors]    = system_characteristics(q_ob_real, q_ob_ref_ws, w_obb, w_ob_ref_ws, REF_TIME, CONTROL_MODE, slew_tol)
end

if SAVE_PLOTS
    FN = add_plots(USE_ESTIMATES, OBSERVERCHOICE, MTQ, CONTROL_MODE, controllerChoice, ENABLE_NOISE, SET_DELAY, UNCERTAIN_WHEELS);
    ws_filename = 'ws.mat';
    full_path = strcat(FN,ws_filename);
    save(full_path);
end







%% Specialization project plots
% Plot orbit
if bool_plot_orbit
    test_length = 850;
    plot_orbit_tail(position, Re, test_length, skip_step, min_tail, wanted_steps);
end

% Plot complete orbit
if bool_plot_complete_orbit
    plot_orbit_complete(position, Re)
end
    
% Plot attitude
if bool_plot_attitude
    test_length_quat = 250;
    plot_attitude(attitude_quaternion, test_length_quat);
end

% Plot attitude with satellite
if bool_plot_attitude_w_satellite
    test_length_quat = 250;
    plot_body_attitude(attitude_quaternion, test_length_quat);
end

