function [settling_times, pointing_accuracy, steady_state_errors] = system_characteristics(q_ob_real, q_ob_ref, w_obb, w_ob_ref, REF_TIME, CONTROL_MODE, tolerance)

settling_times            = zeros(3,1);
pointing_accuracy         = zeros(3,1);
steady_state_errors       = zeros(3,1);

q_ob_real = resample(q_ob_real, q_ob_real.Time(q_ob_real.Time>REF_TIME));
q_ob_ref = resample(q_ob_ref, q_ob_ref.Time(q_ob_ref.Time>REF_TIME));

quat            = q_ob_real.Data;
to_plot_eul     = quat2eul(quat); %eulZYX
to_plot         = rad2deg(to_plot_eul);

ref_eul         = quat2eul(q_ob_ref.Data);
ref             = rad2deg(ref_eul);

deg_serie = timeseries(to_plot, q_ob_real.Time);
deg_ref_serie = timeseries(ref, q_ob_ref.Time);
% 
% figure('units','normalized','outerposition',[0 0 1 1])
% 
% plot(q_ob_real.Time, q_ob_real.Data(:,2:end), 'LineWidth', 2)

if CONTROL_MODE == 3
    
    eps_tolerance = 0.5*deg2rad(tolerance);

    for i=2:4
        [settling_times(i-1)]   = settling_time(q_ob_real.Data(:,i), q_ob_real.Time, q_ob_ref.Data(:,i), REF_TIME, eps_tolerance);
        [pointing_accuracy(i-1), steady_state_errors(i-1)]  = accuracy(q_ob_real, q_ob_ref, settling_times(i-1)+REF_TIME, i);
    end
    pointing_accuracy = rad2deg(2*pointing_accuracy);
    steady_state_errors = rad2deg(2*steady_state_errors);
elseif CONTROL_MODE == 1
    w_obb_tolerance = tolerance;

    for i=1:3
        [settling_times(i)]   = settling_time(w_obb.Data(:,i), w_obb.Time, w_ob_ref.Data(:,i), REF_TIME, w_obb_tolerance);
        [pointing_accuracy(i), steady_state_errors(i)]  = accuracy(w_obb, w_ob_ref, settling_times(i)+REF_TIME, i);
    end
end
end

function [t_set] = settling_time(result, time, ref, REF_TIME, tol)


% SetTimeV1=find(diff(result(:,1))>0.05)+1
% SetTimeV2=find(diff(result(:,2))>0.05)+1
% SetTimeV3=find(diff(result(:,3))>0.05)+1

indices = find(result<=ref+tol & result >= ref-tol);

if indices(end) == length(result)
%     tol_indices = find(diff(indices)>1);
%     tol_indices = [0;tol_indices];
%     index = indices(tol_indices(end)+1);
%     t_set = time(index)-REF_TIME;
    D = diff([0,diff(indices')==1,0]);
    firsts = indices(D>0);
    index = firsts(end);
    t_set = time(index)-REF_TIME;
else
    t_set = time(end);
end
end

function [acc, ss_error] = accuracy(q_ob_real, q_ob_ref, settling_time, i)
% Using RMS, returning pointing accuracy, steady-state error
if ~any(q_ob_real.Time>settling_time)
    acc = -1;
else
    q_ob_real_settled = resample(q_ob_real, q_ob_real.Time(q_ob_real.Time>settling_time));
    q_ob_ref_settled = resample(q_ob_ref, q_ob_ref.Time(q_ob_ref.Time>settling_time));
    acc = rms(q_ob_real_settled.Data(:,i)-q_ob_ref_settled.Data(:,i));
end
last_value = getdatasamples(q_ob_real,length(q_ob_real.Time));
last_ref = getdatasamples(q_ob_ref,length(q_ob_ref.Time));
ss_error = abs(last_ref(i)-last_value(i));
end


